@@ -1,17 +1,15 @@
 from __future__ import annotations
 
 import enum
-import functools
 from functools import wraps
 from typing import Any
 
 from wrapt import ObjectProxy
 
-from coredis.exceptions import ClusterDownError, RedisClusterException
+from coredis.exceptions import ClusterDownError
 from coredis.typing import (
     Callable,
     Coroutine,
-    Dict,
     Iterable,
     List,
     Mapping,
@@ -96,23 +94,23 @@ class EncodingInsensitiveDict(ObjectProxy):  # type: ignore
         return repr(self.__wrapped__)
 
 
-def b(x: ResponseType) -> bytes:
+def b(x: ResponseType, encoding: Optional[str] = None) -> bytes:
     if isinstance(x, bytes):
         return x
     if not isinstance(x, str):
         _v = str(x)
     else:
         _v = x
-    return _v.encode("latin-1")
+    return _v.encode(encoding) if encoding else _v.encode()
 
 
 def defaultvalue(value: Optional[U], default: T) -> Union[U, T]:
     return default if value is None else value
 
 
-def nativestr(x: ResponseType) -> str:
+def nativestr(x: ResponseType, encoding: str = "utf-8") -> str:
     if isinstance(x, (str, bytes)):
-        return x if isinstance(x, str) else x.decode("utf-8", "replace")
+        return x if isinstance(x, str) else x.decode(encoding, "replace")
     elif isinstance(x, (int, float, bool)):
         return str(x)
     raise ValueError(f"Unable to cast {x} to string")
@@ -144,20 +142,7 @@ def dict_to_flat_list(
     return ret
 
 
-# ++++++++++ result callbacks helpers ++++++++++++++
-
-
-def first_key(res: Dict[str, R]) -> R:
-    """
-    Returns the first result for the given command.
-
-    If more then 1 result is returned then a `RedisClusterException` is raised.
-    """
-
-    if len(res.keys()) != 1:
-        raise RedisClusterException("More then 1 result from command")
-
-    return list(res.values())[0]
+# ++++++++++ cluster utils ++++++++++++++
 
 
 def clusterdown_wrapper(
@@ -477,10 +462,19 @@ except ImportError:
 
 @enum.unique
 class CaseAndEncodingInsensitiveEnum(bytes, enum.Enum):
-    @functools.cached_property
+    __decoded: Set[StringT]
+
+    @property
     def variants(self) -> Set[StringT]:
-        decoded = str(self)
-        return {self.value.lower(), self.value, decoded.lower(), decoded.upper()}
+        if not hasattr(self, "__decoded"):
+            decoded = str(self)
+            self.__decoded = {
+                self.value.lower(),  # type: ignore
+                self.value,  # type: ignore
+                decoded.lower(),
+                decoded.upper(),
+            }
+        return self.__decoded
 
     def __eq__(self, other: object) -> bool:
         """
