@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import asyncio
+import contextlib
+import contextvars
 import functools
 import inspect
 import textwrap
@@ -38,6 +40,7 @@ from coredis.typing import (
     AsyncIterator,
     Awaitable,
     Callable,
+    ContextManager,
     Coroutine,
     Dict,
     Iterable,
@@ -410,6 +413,9 @@ class RedisCluster(
         ] = self.__class__.RESULT_CALLBACKS.copy()
         self.non_atomic_cross_slot = non_atomic_cross_slot
         self.cache = cache
+        self._decodecontext: contextvars.ContextVar[
+            Optional[bool],
+        ] = contextvars.ContextVar("decode", default=None)
 
     @classmethod
     @overload
@@ -714,7 +720,7 @@ class RedisCluster(
                     command,
                     *args,
                     noreply=self.noreply,
-                    decode=kwargs.get("decode"),
+                    decode=kwargs.get("decode", self._decodecontext.get()),
                 )
                 if quick_release and not self.requires_wait:
                     released = True
@@ -822,7 +828,7 @@ class RedisCluster(
                             command,
                             *args,
                             noreply=self.noreply,
-                            decode=options.get("decode"),
+                            decode=options.get("decode", self._decodecontext.get()),
                         )
                         try:
                             reply = await request
@@ -849,7 +855,7 @@ class RedisCluster(
                         command,
                         *args,
                         noreply=self.noreply,
-                        decode=options.get("decode"),
+                        decode=options.get("decode", self._decodecontext.get()),
                         raise_exceptions=False,
                     )
                     reply = await request
@@ -874,7 +880,7 @@ class RedisCluster(
                         command,
                         *args,
                         noreply=self.noreply,
-                        decode=options.get("decode"),
+                        decode=options.get("decode", self._decodecontext.get()),
                         raise_exceptions=False,
                     )
                 except ConnectionError as err:
@@ -897,6 +903,38 @@ class RedisCluster(
             return None  # type: ignore
         return self._merge_result(command, res, **options)
 
+    @overload
+    def decoding(self, mode: Literal[False]) -> ContextManager[RedisCluster[bytes]]:
+        ...
+
+    @overload
+    def decoding(self, mode: Literal[True]) -> ContextManager[RedisCluster[str]]:
+        ...
+
+    @contextlib.contextmanager
+    def decoding(self, mode: bool) -> Iterator[RedisCluster[Any]]:
+        """
+        Context manager to temporarily change the decoding behavior
+        of the client
+
+        Example::
+
+            client = coredis.RedisCluster(decode_responses=True)
+            await client.set("fubar", "baz")
+            assert await client.get("fubar") == "baz"
+            with client.decoding(False):
+                assert await client.get("fubar") == b"baz"
+                with client.decoding(True):
+                    assert await client.get("fubar") == "baz"
+
+        """
+        prev = self._decodecontext.get()
+        self._decodecontext.set(mode)
+        try:
+            yield self
+        finally:
+            self._decodecontext.set(prev)
+
     def pubsub(
         self, ignore_subscribe_messages: bool = False, **kwargs: Any
     ) -> ClusterPubSub[AnyStr]:
