@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any, cast
 
 from deprecated.sphinx import versionadded
 
-from coredis._utils import b, nativestr
+from coredis._utils import b, hash_slot, nativestr
 from coredis.commands.constants import CommandName
 from coredis.connection import BaseConnection, Connection
 from coredis.exceptions import ConnectionError, PubSubError, TimeoutError
@@ -71,7 +71,9 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         if hasattr(self, "encoding"):
             return
 
+        await self.connection_pool.initialize()
         conn = await self.connection_pool.get_connection(b"pubsub")
+
         try:
             self.encoding = conn.encoding
             self.decode_responses = conn.decode_responses
@@ -79,13 +81,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
             self.connection_pool.release(conn)
 
     def __del__(self) -> None:
-        try:
-            # if this object went out of scope prior to shutting down
-            # subscriptions, close the connection manually before
-            # returning it to the connection pool
-            self.reset()
-        except Exception:
-            pass
+        self.reset()
 
     def reset(self) -> None:
         """
@@ -135,9 +131,9 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         """
 
         if self.decode_responses and isinstance(value, bytes):
-            value = value.decode(self.encoding)
+            value = nativestr(value, self.encoding)
         elif not self.decode_responses and isinstance(value, str):
-            value = value.encode(self.encoding)
+            value = b(value, self.encoding)
 
         return value
 
@@ -182,17 +178,12 @@ class BasePubSub(Generic[AnyStr, PoolT]):
             return await command(*args)
         except asyncio.CancelledError:
             # do not retry if coroutine is cancelled
-
-            if await connection.can_read():
-                # disconnect if buffer is not empty in case of error
-                # when connection is reused
+            if await connection.can_read():  # noqa
                 connection.disconnect()
-
             raise
         except (ConnectionError, TimeoutError) as e:
             connection.disconnect()
-
-            if not connection.retry_on_timeout and isinstance(e, TimeoutError):
+            if not connection.retry_on_timeout and isinstance(e, TimeoutError):  # noqa
                 raise
             # Connect manually here. If the Redis server is down, this will
             # fail and raise a ConnectionError as desired.
@@ -200,40 +191,32 @@ class BasePubSub(Generic[AnyStr, PoolT]):
             # the ``on_connect`` callback should haven been called by the
             # connection to resubscribe us to any channels and patterns we were
             # previously listening to
-
             return await command(*args)
 
     async def parse_response(
-        self, block: bool = True, timeout: float = 0.0
+        self, block: bool = True, timeout: Optional[float] = None
     ) -> ResponseType:
         """
         Parses the response from a publish/subscribe command
 
         :meta private:
         """
-        connection = self.connection
-
-        if connection is None:
-            raise RuntimeError(
-                "pubsub connection not set: "
-                "did you forget to call subscribe() or psubscribe()?"
-            )
+        assert self.connection
         coro = self._execute(
-            connection,
+            self.connection,
             partial(
-                connection.read_response,
+                self.connection.fetch_push_message,
                 push_message_types=self.SUBUNSUB_MESSAGE_TYPES
                 | self.PUBLISH_MESSAGE_TYPES,
             ),
         )
 
-        if not block and timeout > 0:
-            try:
-                return await asyncio.wait_for(coro, timeout)
-            except asyncio.TimeoutError:
-                return None
-
-        return await coro
+        try:
+            return await asyncio.wait_for(
+                coro, timeout if (timeout and timeout > 0) else None
+            )
+        except asyncio.TimeoutError:
+            return None
 
     async def psubscribe(
         self,
@@ -319,7 +302,9 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         return None
 
     async def get_message(
-        self, ignore_subscribe_messages: bool = False, timeout: Union[int, float] = 0
+        self,
+        ignore_subscribe_messages: bool = False,
+        timeout: Optional[Union[int, float]] = None,
     ) -> Optional[PubSubMessage]:
         """
         Gets the next message if one is available, otherwise None.
@@ -327,7 +312,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         :param ignore_subscribe_messages: Whether to skip subscription
          acknowledgement messages
         :param timeout: Number of seconds to wait for a message to be available
-         on the connection.
+         on the connection. If the ``None`` the command will block forever.
         """
         response = await self.parse_response(block=False, timeout=timeout)
 
@@ -385,10 +370,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
                 subscribed_dict = self.patterns
             else:
                 subscribed_dict = self.channels
-            try:
-                del subscribed_dict[message["channel"]]
-            except KeyError:
-                pass
+            subscribed_dict.pop(message["channel"])
 
         if message_type in self.PUBLISH_MESSAGE_TYPES:
             handler = None
@@ -493,7 +475,7 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
 
     PUBLISH_MESSAGE_TYPES = {b"message", b"smessage"}
     SUBUNSUB_MESSAGE_TYPES = {b"ssubscribe", b"sunsubscribe"}
-    UNSUBSCRIBE_MESSAGE_TYPES = {b"unsubscribe"}
+    UNSUBSCRIBE_MESSAGE_TYPES = {b"sunsubscribe"}
 
     def __init__(
         self,
@@ -529,9 +511,10 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
 
         for channel, handler in channel_handlers.items():
             new_channels[self.encode(channel)] = handler
-        await self.execute_command(
-            CommandName.SSUBSCRIBE, *new_channels.keys(), sharded=True
-        )
+        for new_channel in new_channels.keys():
+            await self.execute_command(
+                CommandName.SSUBSCRIBE, new_channel, sharded=True
+            )
         self.channels.update(new_channels)
 
     async def unsubscribe(self, *channels: StringT) -> None:
@@ -542,7 +525,8 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
         """
 
         await self._ensure_encoding()
-        await self.execute_command(CommandName.SUNSUBSCRIBE, *channels, sharded=True)
+        for channel in channels or list(self.channels.keys()):
+            await self.execute_command(CommandName.SUNSUBSCRIBE, channel, sharded=True)
 
     async def psubscribe(
         self,
@@ -575,15 +559,15 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
 
         assert isinstance(args[0], (bytes, str))
         channel = nativestr(args[0])
-        slot = self.connection_pool.nodes.keyslot(channel)
+        slot = hash_slot(b(channel))
         node = self.connection_pool.nodes.node_from_slot(slot)
-        if node and node["node_id"]:
-            key = node["node_id"]
+        if node and node.node_id:
+            key = node.node_id
             if self.shard_connections.get(key) is None:
                 self.shard_connections[key] = await self.connection_pool.get_connection(
                     b"pubsub",
                     channel=channel,
-                    node_type="replica" if self.read_from_replicas else "master",
+                    node_type="replica" if self.read_from_replicas else "primary",
                 )
                 # register a callback that re-subscribes to any channels we
                 # were listening to when we were disconnected
@@ -600,7 +584,7 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
         raise PubSubError(f"Unable to determine shard for channel {args[0]!r}")
 
     async def parse_response(
-        self, block: bool = True, timeout: float = 0.0
+        self, block: bool = True, timeout: Optional[float] = None
     ) -> ResponseType:
         if not self.shard_connections:
             raise RuntimeError(
@@ -619,13 +603,12 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
                     task.cancel()
         # If there were no pending results check the shards
         if not result:
-            tasks: Dict[str, asyncio.Task[ResponseType]]
-            if tasks := {
+            tasks: Dict[str, asyncio.Task[ResponseType]] = {
                 node_id: asyncio.create_task(
                     self._execute(
                         connection,
                         partial(
-                            connection.read_response,
+                            connection.fetch_push_message,
                             push_message_types=self.SUBUNSUB_MESSAGE_TYPES
                             | self.PUBLISH_MESSAGE_TYPES,
                         ),
@@ -633,10 +616,11 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
                 )
                 for node_id, connection in self.shard_connections.items()
                 if node_id not in self.pending_tasks
-            }:
+            }
+            if tasks:
                 done, pending = await asyncio.wait(
                     tasks.values(),
-                    timeout=timeout if timeout > 0 else None,
+                    timeout=timeout if (timeout and timeout > 0) else None,
                     return_when=asyncio.FIRST_COMPLETED,
                 )
                 if done:
