@@ -1,7 +1,5 @@
 from __future__ import annotations
 
-from typing import Optional
-
 from coredis.response._callbacks import (
     AnyStrCallback,
     DictCallback,
@@ -15,7 +13,16 @@ from coredis.response._callbacks.sentinel import (
     SentinelInfoCallback,
     SentinelsStateCallback,
 )
-from coredis.typing import AnyStr, Dict, ResponseType, StringT, Tuple, Union, ValueT
+from coredis.typing import (
+    AnyStr,
+    Dict,
+    Optional,
+    ResponseType,
+    StringT,
+    Tuple,
+    Union,
+    ValueT,
+)
 
 from . import CommandMixin
 from ._wrappers import redis_command
@@ -96,13 +103,15 @@ class SentinelCommands(CommandMixin[AnyStr]):
 
     @redis_command(CommandName.SENTINEL_INFO_CACHE)
     async def sentinel_infocache(
-        self,
+        self, *nodenames: StringT
     ) -> Dict[AnyStr, Dict[int, Dict[str, ResponseType]]]:
         """
         Return cached INFO output from masters and replicas.
         """
         return await self.execute_command(
-            CommandName.SENTINEL_INFO_CACHE, callback=SentinelInfoCallback[AnyStr]()
+            CommandName.SENTINEL_INFO_CACHE,
+            *nodenames,
+            callback=SentinelInfoCallback[AnyStr](),
         )
 
     @redis_command(
