@@ -1,9 +1,8 @@
 from __future__ import annotations
 
 import re
-from typing import Any
 
-from coredis.typing import Mapping, Optional, Set, Tuple, ValueT
+from coredis.typing import Optional, Set, Tuple, ValueT
 
 
 class RedisError(Exception):
@@ -66,7 +65,39 @@ class DataError(RedisError):
 
 
 class NoKeyError(RedisError):
-    pass
+    """
+    Raised when a key provided in the command is missing
+    """
+
+
+class ReplicationError(RedisError):
+    """
+    Raised when then replication requirements were not met
+    """
+
+    def __init__(self, command: bytes, replicas: int, timeout: int):
+        super().__init__(
+            f"Command {str(command)} did not replicate to {replicas} replicas within {timeout} ms."
+        )
+
+
+class NotBusyError(ResponseError):
+    """
+    Raised when script kill or function kill have nothing to terminate
+    """
+
+
+class UnblockedError(ResponseError):
+    """
+    Raised if a blocked client is unblocked forcefully
+    """
+
+
+class WrongTypeError(ResponseError):
+    """
+    Raised when an operation is performed on a key
+    containing a datatype that doesn't support the operation
+    """
 
 
 class PubSubError(RedisError):
@@ -121,6 +152,10 @@ class ClusterCrossSlotError(ResponseError):
         self.keys = keys
 
 
+class ClusterRoutingError(RedisClusterException):
+    """Raised when keys in request can't be routed to destination nodes"""
+
+
 class ClusterDownError(ClusterError, ResponseError):
     """
     Error indicated ``CLUSTERDOWN`` error received from cluster.
@@ -149,10 +184,6 @@ class ClusterResponseError(ClusterError):
     cluster responses has errors.
     """
 
-    def __init__(self, message: str, responses: Mapping[str, Any]) -> None:
-        super().__init__(message)
-        self.responses = responses
-
 
 class AskError(ResponseError):
     """
@@ -216,7 +247,7 @@ class AuthenticationRequiredError(AuthenticationError):
     """
 
 
-class AuthorizationError(RedisError):
+class AuthorizationError(ResponseError):
     """
     Base class for authorization errors
     """
@@ -252,18 +283,38 @@ class UnknownCommandError(ResponseError):
     to an unknown command.
     """
 
-    ERROR_REGEX = re.compile("unknown command `(.*?)`")
+    ERROR_REGEX = re.compile("unknown command (.*?)")
     #: Name of command requested
     command: str
 
     def __init__(self, message: str) -> None:
-        self.command = self.ERROR_REGEX.findall(message).pop()
+        command_match = self.ERROR_REGEX.findall(message)
+        if command_match:
+            self.command = command_match.pop()
         super().__init__(self, message)
 
 
 class StreamConsumerError(RedisError):
-    pass
+    """
+    Base exception for stream consumer related errors
+    """
+
+
+class StreamConsumerGroupError(StreamConsumerError):
+    """
+    Base exception for consumer group related errors
+    """
+
+
+class StreamDuplicateConsumerGroupError(StreamConsumerGroupError):
+    """
+    Raised when and attempt to create a stream consumer
+    group fails because it already exists
+    """
 
 
 class StreamConsumerInitializationError(StreamConsumerError):
-    pass
+    """
+    Raised when a stream consumer could not be initialized
+    based on the configuration provided
+    """
